@@ -31,6 +31,11 @@
 #include "pygtk-private.h"
 #include <pyerrors.h>
 
+#ifdef HAVE_PYCAIRO
+# include <pycairo.h>
+Pycairo_CAPI_t *Pycairo_CAPI;
+#endif
+
 void _pygtk_register_boxed_types(PyObject *moddict);
 void pygtk_register_classes(PyObject *d);
 void pygdk_register_classes(PyObject *d);
@@ -43,6 +48,7 @@ extern PyMethodDef pygdk_functions[];
 PyObject *PyGtkDeprecationWarning;
 PyObject *PyGtkWarning;
 
+
 static struct _PyGtk_FunctionStruct functions = {
     VERSION,
 
@@ -118,7 +124,12 @@ init_gtk(void)
     char **argv;
     GSList *stock_ids, *cur;
     char buf[128];
- 
+
+#ifdef HAVE_PYCAIRO
+    Pycairo_IMPORT;
+    if (Pycairo_CAPI == NULL)
+        return;
+#endif
     /* initialise gobject */
     init_pygobject();
     g_assert(pygobject_register_class != NULL);
@@ -133,7 +144,20 @@ init_gtk(void)
     /* initialise GTK ... */
     av = PySys_GetObject("argv");
     if (av != NULL) {
-	argc = PyList_Size(av);
+	if (!PyList_Check(av)) {
+	    PyErr_Warn(PyExc_Warning, "ignoring sys.argv: it must be a list of strings");
+	    av = NULL;
+	} else {
+	    argc = PyList_Size(av);
+	    for (i = 0; i < argc; i++)
+		if (!PyString_Check(PyList_GetItem(av, i))) {
+		    PyErr_Warn(PyExc_Warning, "ignoring sys.argv: it must be a list of strings");
+		    av = NULL;
+		    break;
+		}
+	}
+    }
+    if (av != NULL) {
 	argv = g_new(char *, argc);
 	for (i = 0; i < argc; i++)
 	    argv[i] = g_strdup(PyString_AsString(PyList_GetItem(av, i)));
@@ -149,14 +173,18 @@ init_gtk(void)
 	    g_free(argv);
 	}
 	PyErr_SetString(PyExc_RuntimeError, "could not open display");
-	/* set the LC_NUMERIC locale back to "C", as Python requires
+	/* set the LC_NUMERIC locale back to "C", as Python < 2.4 requires
 	 * that it be set that way. */
+#if PY_VERSION_HEX < 0x020400F0
 	setlocale(LC_NUMERIC, "C");
+#endif
 	return;
     }
-    /* set the LC_NUMERIC locale back to "C", as Python requires that
+    /* set the LC_NUMERIC locale back to "C", as Python < 2.4 requires that
      * it be set that way. */
+#if PY_VERSION_HEX < 0x020400F0
     setlocale(LC_NUMERIC, "C");
+#endif
     if (argv != NULL) {
 	PySys_SetArgv(argc, argv);
 	for (i = 0; i < argc; i++)
